using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Telerik.WebControls;

public partial class ServerSideCS : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {

    }

    // first the date selection is performed on the client (and then the postback to the server is triggered)
    // that is why we need to process the proper day(s) selection depending on our goals
    protected void RadCalendar1_SelectionChanged(object sender, Telerik.WebControls.Base.Calendar.Events.SelectedDatesEventArgs e)
    {
        int selectedDatesCount = RadCalendar1.SelectedDates.Count;
        DateTime startWeekDate = DateTime.MinValue;
        DateTime endWeekDate = DateTime.MinValue;
        // do nothing if there is a single selected date, a whole selected week or no selected dates
        if (selectedDatesCount < 2 || selectedDatesCount == 7)
        {
            return;
        }
        //a date cell is clicked, clear the old selected date
        if (selectedDatesCount == 2)
        {
            RadCalendar1.SelectedDates.RemoveAt(0);
        }
        // first a week is selected, than a day from the same week
        // or vice versa
        else if (selectedDatesCount == 6)
        {
            //SelectedDates is a queue but we need the dates inside sorted to compare them
            RadCalendar1.SelectedDates.Sort(new DateComparer());
            
            // get the first and last selected dates from week
            startWeekDate = RadCalendar1.SelectedDates[0].Date;
            endWeekDate = RadCalendar1.SelectedDates[RadCalendar1.SelectedDates.Count - 1].Date;

            // get the first day of week depending of the calendar DateTimeFormat and culture
            DayOfWeek firstDayOfWeek = RadCalendar1.DateTimeFormat.FirstDayOfWeek;
            // get the integer corresponding to the last week day (to use it with the DayOfWeek enumeration)
            int lastDayOfWeekNumber = ((int)firstDayOfWeek + 6) % 7;
            DayOfWeek lastDayOfWeek = (DayOfWeek)Enum.Parse(typeof(DayOfWeek), lastDayOfWeekNumber.ToString());

            // searching for the last clicked day - it is not inside the SelectedDates collection,
            // so we need to determine it based on the DayOfWeek property of the currently SelectedDates
            DateTime dateToSelect = DateTime.MinValue;
            if (RadCalendar1.Calendar.GetDayOfWeek(startWeekDate) != firstDayOfWeek) //the first week day was clicked
            {
                dateToSelect = startWeekDate.AddDays(-1);
            }
            else if (RadCalendar1.Calendar.GetDayOfWeek(endWeekDate) != lastDayOfWeek) //the last week day was clicked
            {
                dateToSelect = endWeekDate.AddDays(1);
            }
            //loop through the rest of the week days until the clicked day is found (its current state 
            //is unselected and there would be a difference of two days when we calculate the difference 
            // between each pair of consecutive days in the SelectedDates collection
            else 
            {
                for (int i = 0; i < selectedDatesCount - 1; i++)
                {   
                    if (RadCalendar1.SelectedDates[i + 1].Date.Day - RadCalendar1.SelectedDates[i].Date.Day > 1)
                    {
                        dateToSelect = RadCalendar1.SelectedDates[i].Date.AddDays(1);
                        break;
                    }
                }
            }
            RadCalendar1.SelectedDates.Clear();
            RadCalendar1.SelectedDates.Add(new RadDate(dateToSelect));
        }
        else if (selectedDatesCount == 8) //selected day and a week - one should be unselected, depending on SelectedDates sequence
        {   
            //SelectedDates collection is a queue:
            //if first the day is selected, than the week - the Day is first in the SelectedDates collection => the day should be removed (1)
            //if the week is selected first, than the day - the Day is last in the SelectedDates collection => the week should be removed (2)
            
            int weekNumber = GetWeekNumberFromDate(RadCalendar1.SelectedDates[0].Date);
            int dayWeekNumber = GetWeekNumberFromDate(RadCalendar1.SelectedDates[1].Date);

            if (weekNumber != dayWeekNumber) // (1) 
            {
                RadCalendar1.SelectedDates.RemoveAt(0);
    
            }
            else // (2)
            {
                DateTime selectedDate = RadCalendar1.SelectedDates[RadCalendar1.SelectedDates.Count - 1].Date;
                RadCalendar1.SelectedDates.Clear();
                RadCalendar1.SelectedDates.Add(new RadDate(selectedDate));
            }
        }
        else //selectedDates are two weeks - only the last selected one should remain
        {
            endWeekDate = RadCalendar1.SelectedDates[RadCalendar1.SelectedDates.Count - 1].Date;
            startWeekDate = endWeekDate.AddDays(-6);

            List<int> indexesToClear = new List<int>();
            for (int i = 0; i < RadCalendar1.SelectedDates.Count; i++)
            {
                if (RadCalendar1.SelectedDates[i].Date < startWeekDate || RadCalendar1.SelectedDates[i].Date > endWeekDate)
                {
                    indexesToClear.Add(i);
                }
            }
            for (int j = indexesToClear.Count - 1; j >= 0; j--)
            {
                RadCalendar1.SelectedDates.RemoveAt(j);
            } 
        }
    }
    
    private int GetWeekNumberFromDate(DateTime date)
    {
        // get the first day of week depending of the calendar DateTimeFormat and culture
        DayOfWeek firstDayOfWeek = RadCalendar1.DateTimeFormat.FirstDayOfWeek;
        // get the integer corresponding to the last week day (to use it with the DayOfWeek enumeration)
        int lastDayOfWeekNumber = ((int)firstDayOfWeek + 6) % 7;
        
        int daysToAdd = lastDayOfWeekNumber - (int)date.DayOfWeek;
        DateTime lastWeekDay = date.AddDays(daysToAdd);
        return RadCalendar1.Calendar.GetWeekOfYear(lastWeekDay, RadCalendar1.DateTimeFormat.CalendarWeekRule, firstDayOfWeek);
    }
}

public class DateComparer : IComparer
{
    public int Compare(object x, object y)
    {
        if ((x as RadDate).Date < (y as RadDate).Date)
        {
            return -1;
        }
        else if ((x as RadDate).Date > (y as RadDate).Date)
        {
            return 1;
        }
        else
        {
            return 0;
        }
    }
}
