Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports Telerik.WebControls

Partial Class ServerSideVB
    Inherits System.Web.UI.Page

    ' first the date selection is performed on the client (and then the postback to the server is triggered)
    ' that is why we need to process the proper day(s) selection depending on our goals
    Protected Sub RadCalendar1_DayRender(ByVal sender As Object, ByVal e As Telerik.WebControls.Base.Calendar.Events.DayRenderEventArgs) Handles RadCalendar1.DayRender

        Dim selectedDatesCount As Integer = RadCalendar1.SelectedDates.Count
        Dim startWeekDate As DateTime = DateTime.MinValue
        Dim endWeekDate As DateTime = DateTime.MinValue
        ' do nothing if there is a single selected date, a whole selected week or no selected dates
        If ((selectedDatesCount < 2) Or (selectedDatesCount = 7)) Then
            Return
        End If
        'a date cell is clicked, clear the old selected date
        If (selectedDatesCount = 2) Then
            RadCalendar1.SelectedDates.RemoveAt(0)
            ' first a week is selected, than a day from the same week
            ' or vice versa
        ElseIf (selectedDatesCount = 6) Then
            'SelectedDates is a queue but we need the dates inside sorted to compare them
            RadCalendar1.SelectedDates.Sort(New DateComparer)
            ' get the first and last selected dates from week
            startWeekDate = RadCalendar1.SelectedDates(0).Date
            endWeekDate = RadCalendar1.SelectedDates((RadCalendar1.SelectedDates.Count - 1)).Date
            ' get the first day of week depending of the calendar DateTimeFormat and culture
            Dim firstDayOfWeek As DayOfWeek = RadCalendar1.DateTimeFormat.FirstDayOfWeek
            ' get the integer corresponding to the last week day (to use it with the DayOfWeek enumeration)
            Dim lastDayOfWeekNumber As Integer = ((CType(firstDayOfWeek, Integer) + 6) Mod 7)
            Dim lastDayOfWeek As DayOfWeek = CType(System.Enum.Parse(GetType(DayOfWeek), lastDayOfWeekNumber.ToString), DayOfWeek)
            ' searching for the last clicked day - it is not inside the SelectedDates collection,
            ' so we need to determine it based on the DayOfWeek property of the currently SelectedDates
            Dim dateToSelect As DateTime = DateTime.MinValue
            If (RadCalendar1.Calendar.GetDayOfWeek(startWeekDate) <> firstDayOfWeek) Then
                dateToSelect = startWeekDate.AddDays(-1)
            ElseIf (RadCalendar1.Calendar.GetDayOfWeek(endWeekDate) <> lastDayOfWeek) Then
                dateToSelect = endWeekDate.AddDays(1)
            End If
            'loop through the rest of the week days until the clicked day is found (its current state 
            'is unselected and there would be a difference of two days when we calculate the difference 
            ' between each pair of consecutive days in the SelectedDates collection
            Dim i As Integer = 0
            Do While (i < (selectedDatesCount - 1))
                If ((RadCalendar1.SelectedDates((i + 1)).Date.Day - RadCalendar1.SelectedDates(i).Date.Day) > 1) Then
                    dateToSelect = RadCalendar1.SelectedDates(i).Date.AddDays(1)
                    Exit Do
                End If
                i = (i + 1)
            Loop
            RadCalendar1.SelectedDates.Clear()
            RadCalendar1.SelectedDates.Add(New RadDate(dateToSelect))
        ElseIf (selectedDatesCount = 8) Then
            'SelectedDates collection is a queue:
            'if first the day is selected, than the week - the Day is first in the SelectedDates collection => the day should be removed (1)
            'if the week is selected first, than the day - the Day is last in the SelectedDates collection => the week should be removed (2)
            Dim weekNumber As Integer = GetWeekNumberFromDate(RadCalendar1.SelectedDates(0).Date)
            Dim dayWeekNumber As Integer = GetWeekNumberFromDate(RadCalendar1.SelectedDates(1).Date)
            If (weekNumber <> dayWeekNumber) Then
                RadCalendar1.SelectedDates.RemoveAt(0)
            Else
                Dim selectedDate As DateTime = RadCalendar1.SelectedDates((RadCalendar1.SelectedDates.Count - 1)).Date
                RadCalendar1.SelectedDates.Clear()
                RadCalendar1.SelectedDates.Add(New RadDate(selectedDate))
            End If
        Else
            endWeekDate = RadCalendar1.SelectedDates((RadCalendar1.SelectedDates.Count - 1)).Date
            startWeekDate = endWeekDate.AddDays(-6)
            Dim indexesToClear As List(Of Integer) = New List(Of Integer)
            Dim i As Integer = 0
            Do While (i < RadCalendar1.SelectedDates.Count)
                If ((RadCalendar1.SelectedDates(i).Date < startWeekDate) _
                            OrElse (RadCalendar1.SelectedDates(i).Date > endWeekDate)) Then
                    indexesToClear.Add(i)
                End If
                i = (i + 1)
            Loop
            Dim j As Integer = (indexesToClear.Count - 1)
            Do While (j >= 0)
                RadCalendar1.SelectedDates.RemoveAt(j)
                j = (j - 1)
            Loop
        End If


    End Sub

    Private Function GetWeekNumberFromDate(ByVal d As DateTime) As Integer
        ' get the first day of week depending of the calendar DateTimeFormat and culture
        Dim firstDayOfWeek As DayOfWeek = RadCalendar1.DateTimeFormat.FirstDayOfWeek
        ' get the integer corresponding to the last week day (to use it with the DayOfWeek enumeration)
        Dim lastDayOfWeekNumber As Integer = ((CType(firstDayOfWeek, Integer) + 6) Mod 7)
        Dim daysToAdd As Integer = (lastDayOfWeekNumber - CType(d.DayOfWeek, Integer))
        Dim lastWeekDay As DateTime = d.AddDays(daysToAdd)
        Return RadCalendar1.Calendar.GetWeekOfYear(lastWeekDay, RadCalendar1.DateTimeFormat.CalendarWeekRule, firstDayOfWeek)
    End Function

End Class

Public Class DateComparer
    Implements IComparer

    Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements System.Collections.IComparer.Compare
        If (CType(x, RadDate).Date < CType(y, RadDate).Date) Then
            Return -1
        ElseIf (CType(x, RadDate).Date > CType(y, RadDate).Date) Then
            Return 1
        Else
            Return 0
        End If
    End Function

End Class
